/*
 * Copyright 2022 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

import QtQuick 2.15
import QtQuick.Controls 2.15
import QtQml 2.15
import QtGraphicalEffects 1.15
import com.kylin.tablet 1.0
import "."

Item {
    id: root
    property int iconState: LauncherItem.Normal
    property int appId: 0
    property int pageIndex: 0

    property bool selected: true;

    property string icon;
    property int iconWidth: 100;
    property int iconRadius: 10;

    property int deleteiconWidth: 30;
    property int selecticonWidth: 30;

    signal unload(var appid);
    signal checked(var appid);
    signal unchecked(var appid);

    property bool disableDelete : false

    signal deleteIcon()
    property bool allowDelete: false

    state: {
        switch (iconState) {
        case LauncherItem.Normal: return "Normal";
        case LauncherItem.Editable: return "Editable";
        case LauncherItem.Selectable: return "Selectable";
        }
    }

    states: [
        State {
            name: "Normal"
            StateChangeScript {
                script: {
                    if (deleteiconAnimation.running)
                        deleteiconAnimation.stop();
                    if (appiconAnimation.running)
                        appiconAnimation.stop();
                    if (selecticonAnimation.running)
                        selecticonAnimation.stop();
                    deleteicon.scale = 0;
                    selecticon.scale = 0;
                    appicon.rotation = 0;
                    deleteicon.rotation = 0;
                    if(modelData.type === LauncherItem.AddIcon) {
                        image.width = (iconWidth/2) * Style.tabletScaleFactor
                        image.height = (iconWidth/2) * Style.tabletScaleFactor
                        mask.visible = true
                    }
                }
            }
        },
        State {
            name: "Editable"
            StateChangeScript {
                script: {
                    deleteiconAnimation.start();
                    appiconAnimation.start();
                }
            }
        },
        State {
            name: "Selectable"
            StateChangeScript {
                script: {
                    selecticonAnimation.start(); }
            }
        }
    ]

    width: iconWidth
    height: iconWidth

    Image {
        id: appicon//image;
        source: icon;
        anchors.centerIn: root;
        width: iconWidth
        height: iconWidth
    }
    DropShadow {
        anchors.fill: appicon//image
        horizontalOffset: iconWidth/32
        radius: iconWidth/32
        samples: iconWidth/32 * 2 + 1
        color: "#000000"
        opacity: 0.15
        source: appicon//image
    }
//    OpacityMask {
//        id: appicon;
//        anchors.fill: image;
//        source: image;
//        maskSource: mask;
//    }

//    InnerShadow {
//        anchors.fill: image
//        radius: 4.0
//        samples: 8
//        horizontalOffset: -1
//        verticalOffset: -1
//        color: "#b0000000"
//        source: image
//    }

    Image {
        id: deleteicon;
        visible: true//!disableDelete
        anchors {
            right: appicon.right; rightMargin: -(deleteiconWidth/4) * Style.tabletScaleFactor;
            top: appicon.top; topMargin: -(deleteiconWidth/4) * Style.tabletScaleFactor;
        }
        source: "qrc:/ui/images/desktop-uninstallbtn.png"
        width: deleteiconWidth * Style.tabletScaleFactor
        height: deleteiconWidth * Style.tabletScaleFactor
        antialiasing: true
        MouseArea {
            id: deleteMouseArea
            anchors.fill: parent
            onClicked: {
                deleteIcon()
            }
        }
    }
    ScaleAnimator {
        id: deleteiconAnimation;
        target: deleteicon;
        from: 0;
        to: 1;
        duration: 500;
    }
    SequentialAnimation {
        id: appiconAnimation;
        loops: AnimatedSprite.Infinite;
        ParallelAnimation {
            RotationAnimation {
                target: appicon;
                from: 0;
                to: 2;
                duration: 150;
            }
            RotationAnimation {
                target: deleteicon;
                from: 0;
                to: 5;
                duration: 120;
            }
        }
        ParallelAnimation {
            RotationAnimation {
                target: appicon;
                from: 2;
                to: -2;
                duration: 150;
            }
            RotationAnimation {
                target: deleteicon;
                from: 2;
                to: -5;
                duration: 120;
            }
        }
    }

    Binding { target: selecticon; property: "selected"; value: root.selected }

    Rectangle {
        property bool selected: false

        id: selecticon;
        width: selecticonWidth * Style.widthScaleFactor;
        height: selecticonWidth * Style.heightScaleFactor;
        radius: selecticon.width/2;
        color: "transparent";
        scale: 0;
        anchors.left: appicon.right;
        anchors.leftMargin: (-selecticonWidth/2) * Style.widthScaleFactor;
        anchors.top: appicon.top;
        anchors.topMargin: (-selecticonWidth/5) * Style.heightScaleFactor;
        Image {
            id: selecticonImage;
            source: selecticon.selected ? "qrc:/ui/images/ukui_choice_click" :
                             "qrc:/ui/images/ukui_choice_default";
            anchors.centerIn: parent;
        }
        MouseArea {
            anchors.fill: parent;
            onClicked: {
                selecticon.selected = !selecticon.selected
                modelManager.setItemSelectState(appId, pageIndex, selecticon.selected)
            }
        }
    }
    ScaleAnimator {
        id: selecticonAnimation;
        target: selecticon;
        from: 0;
        to: 1;
        duration: 500;
    }

    Component.onCompleted: {
        if(iconState === LauncherItem.Selectable) {
            selecticonAnimation.start();
            deleteicon.visible = false;
        }
    }
}
