/*
 * Copyright 2022 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

import QtQuick 2.15
import QtQuick.Controls 2.5
import QtQuick.Window 2.0
import QtQml.Models 2.2
import QtQuick.Window 2.2
import QtQuick.Layouts 1.3
import QtQml 2.15
import com.kylin.tablet 1.0
import "."

Item {
    id: root
    property var viewMoel: null
    property bool dragEnable: true
    property real gridCellWidth: Style.tabletScaleFactor * 254
    property real gridCellHeight: Style.tabletScaleFactor * 206
    property real cellContentMargin: Style.tabletScaleFactor * 20
    property Item pluginAnchorItem: undefined
    property var rootWindow: undefined
    property var pageContainer: listView
    property int groupId: -1
    property var appImage
    property var apptext

    signal startDrag(bool started)
    signal showAddAppWindow(int groupId, int pageIndex)
    signal titleFocusChanged()

    function listViewItemCount() {
        return listView.count
    }

    clip: false

    ListView {
        id: listView
        signal movingChanged(bool m)
        anchors.fill: parent
        spacing: 40
        snapMode: ListView.SnapOneItem
        model: viewMoel
        delegate: listViewItemDelegate
        orientation: ListView.Horizontal
        highlightRangeMode: ListView.StrictlyEnforceRange
        highlightFollowsCurrentItem: true
        highlightMoveDuration: 100
        maximumFlickVelocity: 8000
        cacheBuffer: listView.width * listView.count
        clip: false
        focus: true
        onHorizontalVelocityChanged: movingChanged(horizontalVelocity)
    }
    Component {
        id: listViewItemDelegate
        Item {
            width: listView.width
            height: listView.height
//            KGridView {
            GridView {
                id: gridView
                width: Screen.width > Screen.height ?
                           240 * 6 * Style.tabletScaleFactor + 1 :
                           240 * 4 * Style.tabletScaleFactor + 1
                y: 75 * Style.tabletScaleFactor
                height: parent.height
                anchors.horizontalCenter: parent.horizontalCenter
                cellWidth: Style.appIconGridWidth//gridCellWidth
                cellHeight: Style.appIconGridHeight//gridCellHeight
                model: modelData
                interactive: false
                delegate: gridviewItemDelegate
                add: Transition { NumberAnimation { properties: "x,y"; duration: 200 } }
                addDisplaced: Transition { NumberAnimation { properties: "x,y"; duration: 200 } }
                move: Transition { NumberAnimation { properties: "x,y"; duration: 200 } }
                moveDisplaced: Transition { NumberAnimation { properties: "x,y"; duration: 200 } }
                remove: Transition { NumberAnimation { properties: "x,y"; duration: 200 } }
                removeDisplaced: Transition { NumberAnimation { properties: "x,y"; duration: 200 } }
                displaced: Transition { NumberAnimation { properties: "x,y"; duration: 200 } }

                MouseArea {
                    anchors.fill: parent
                    z: -1
                    onClicked: {
                        //do to edit focus
                        dumpInfoBox.visible = false
                        modelManager.setItemState(LauncherItem.Normal)
                        modelManager.setItemState(LauncherItem.Normal,0)
                    }
                    onPressAndHold: {
                        startDrag(true)
                        modelManager.setItemState(LauncherItem.Editable)
                        modelManager.setItemState(LauncherItem.Editable,0)
                    }
                }
            }
        }

    }

    Component {
        id: gridviewItemDelegate
        Item {
            id: delegateContent
            width: modelData.columnSpan *  Style.appIconGridWidth//gridCellWidth //180
            height: modelData.rowSpan *  Style.appIconGridHeight//gridCellHeight //105
            opacity: 1

            property int group: 2
            property int visualIndex: index
            property int pageIndex: modelData.pageIndex
            property int rowSpan: modelData.columnSpan
            property int columnSpan: modelData.columnSpan
            Binding { target: itemRoot; property: "visualIndex"; value: visualIndex }
            Binding { target: itemRoot; property: "pageIndex"; value: pageIndex }
            Binding { target: itemRoot; property: "rowSpan"; value: rowSpan }
            Binding { target: itemRoot; property: "columnSpan"; value: columnSpan }

            Component.onCompleted: {
                itemRoot.visualIndex = index
            }

            Item {
                id: itemRoot
                width: modelData.columnSpan * Style.appIconGridWidth
                height: modelData.rowSpan * Style.appIconGridHeight
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.top: parent.top
                clip: false

                DropArea {
                    anchors.horizontalCenter: parent.horizontalCenter
                    width: item.width
                    height: item.height

                    onEntered: {
                        if (!dragEnable || modelData.type === LauncherItem.AddIcon) {
                            return
                        }
                        if (drag.source.visualIndex === undefined) {
                            return
                        }
                        if (drag.source.pageIndex !== pageIndex || drag.source.group !== group) {
                            if(drag.source.group !== group) {
                                taskbar.taskbarReplayAllowed = true
                            }
                            modelManager.addPlaceholderItem(
                                        visualIndex, pageIndex,
                                        drag.source.rowSpan,
                                        drag.source.columnSpan)
                        } else {
                            modelManager.moveItem(drag.source.visualIndex, visualIndex, pageIndex)
                            modelManager.removePlaceholderItem();
                        }
                    }
                }

                // 方便在拖动时获取数据
                property int visualIndex: 0
                property int pageIndex: 0
                property int rowSpan: 1
                property int columnSpan: 1
                property int group: 2

                Drag.active: dragMousearea.drag.active && dragEnable
                Drag.source: itemRoot
                Drag.hotSpot.x: itemRoot.width / 2
                Drag.hotSpot.y: itemRoot.height / 2

                states: [
                    State {
                        when: itemRoot.Drag.active
                        ParentChange {
                            target: itemRoot
                            parent: taskbar
                        }

                        AnchorChanges {
                            target: itemRoot
                            anchors.horizontalCenter: undefined
                            anchors.top: undefined
                        }
                    }
                ]

                MouseArea {
                    id: dragMousearea
                    anchors.centerIn: parent
                    width: modelData.columnSpan * 130
                    height: modelData.columnSpan * 130
                    hoverEnabled: true
                    Drag.active: false
                    drag.target: undefined
                    pressAndHoldInterval: modelData.state === LauncherItem.Editable ? 100 : undefined
                    property bool isScrolling: false

                    Timer {
                        id: scrollingTimer
                        interval: 600
                        onTriggered: dragMousearea.isScrolling = false
                    }

                    onPressAndHold: {
                        modelManager.setItemState(LauncherItem.Editable)
                        modelManager.setItemState(LauncherItem.Editable,0)

                        dragMousearea.Drag.active = true
                        dragMousearea.drag.target = itemRoot
                        startDrag(true)
                    }
                    onClicked: {
                        if(modelData.state === LauncherItem.Normal){
                            appManager.runApplication(modelData.desktopFile)
                        }
                        root.titleFocusChanged()
                    }
                    onPositionChanged:  {
                        if(taskbar.replayAllowed) {
                            appIcon.iconWidth = 64 * Style.tabletScaleFactor
                        }
                        else {
                            appIcon.iconWidth = 96 * Style.tabletScaleFactor
                        }

                        var itemPos = item.mapToItem(listView, 0, 0)
                        if ((itemPos.x % listView.width) >= (listView.width - item.width/2  - 48 * Style.tabletScaleFactor) &&
                                dragMousearea.Drag.active &&!isScrolling ) {
                            if (listView.currentIndex < listView.count - 1) {
                                listView.currentIndex ++
                                isScrolling = true
                                scrollingTimer.start()
                                modelManager.removePlaceholderItem()
                                // 拖动至不同页时才添加 placeholder item
                                if (drag.target.pageIndex !== listView.currentIndex) {
                                    var curPageItemCount = modelManager.groupPageItemCount(-1, listView.currentIndex);
                                    var index = curPageItemCount > 0 ? curPageItemCount : 0
                                    modelManager.addPlaceholderItem(index, listView.currentIndex, drag.target.rowSpan,
                                                drag.target.columnSpan)
                                }
                            } else {
                                item.x = 0
                            }
                        } else if ((itemPos.x % listView.width < -item.width/2 + 48 * Style.tabletScaleFactor) &&
                                   dragMousearea.Drag.active && !isScrolling) {
                            if (listView.currentIndex > 0) {
                                listView.currentIndex --
                                isScrolling = true
                                scrollingTimer.start()
                                modelManager.removePlaceholderItem()
                                if (drag.target.pageIndex !== listView.currentIndex) {
                                    curPageItemCount = modelManager.groupPageItemCount(-1, listView.currentIndex);
                                    index = curPageItemCount > 0 ? curPageItemCount : 0
                                    modelManager.addPlaceholderItem(index, listView.currentIndex, drag.target.rowSpan,
                                                drag.target.columnSpan, groupId)
                                }
                            }
                        }
                    }
                    onReleased: {
                        appIcon.iconWidth = 96 * Style.tabletScaleFactor

                        isScrolling = false
                        if (dragMousearea.Drag.active) {
                            dragMousearea.Drag.active = false
                            dragMousearea.drag.target = null
                            if(taskbar.replayAllowed) {
                                appIcon.visible = false
                                itemName.visible = false
                                modelManager.replacePlaceholderItemTaskbar(visualIndex, pageIndex, -1, 0)
                                taskbar.replayAllowed = false
                            } else {
                                modelManager.replacePlaceholderItem(visualIndex, pageIndex)
                            }
                            modelManager.updateCachedData(groupId)
                            modelManager.updateCachedData(0)
                        }
                    }
                }
                Rectangle {
                    id: item
                    property bool moving: listView.horizontalVelocity
                    width: modelData.columnSpan * Style.appIconGridWidth/*gridCellWidth*/ - Style.tabletScaleFactor * 40
                    height: modelData.rowSpan * Style.appIconGridHeight/*gridCellHeight*/ - Style.tabletScaleFactor * 40
                    color: "transparent"
                    clip: true
                    radius: 5
                    anchors {
                        horizontalCenter: parent.horizontalCenter;
                        verticalCenter: parent.verticalCenter
                    }
                    AppIcon {
                        id: appIcon
                        iconWidth: Style.tabletScaleFactor * 96/// 2
                        visible: modelData.type === LauncherItem.AppIcon
                        disableDelete: !modelManager.groupEditable(root.groupId)
                        anchors.horizontalCenter: parent.horizontalCenter;
                        anchors.verticalCenter: parent.verticalCenter
                        deleteiconWidth: 40

                        opacity: {
                            if (modelData.type === LauncherItem.AppIcon) {
                                if (listView.moving || dragMousearea.Drag.active) {
                                    return 1
                                }
                                if (dragMousearea.pressed && !dragMousearea.Drag.active) {
                                    return 0.5
                                }
                            }
                            return 1
                        }
                        icon: {
                            switch (modelData.type) {
                            case LauncherItem.AppIcon: return "file://" + modelData.icon
                            case LauncherItem.AddIcon:
                                if (dragMousearea.pressed) {
                                    return "qrc:/ui/images/list-add-click.svg";
                                } else {
                                    return "qrc:/ui/images/list-add-default.svg";
                                }
                            default: return ""
                            }
                        }
                        iconState: modelData.state
                        anchors {
                            horizontalCenter: parent.horizontalCenter
                            verticalCenter: parent.verticalCenter
                        }
                        onDeleteIcon: {
                            appIcon.visible = true
                            itemName.visible = true

                            dumpInfoBox.visible = true
                            dumpInfoBox.desktopFile = modelData.desktopFile
                            dumpInfoBox.visualIndex = itemRoot.visualIndex
                            dumpInfoBox.pageIndex = itemRoot.pageIndex
                            dumpInfoBox.groupId = groupId

                            taskbar.textPromptBox = model.modelData.name
                            taskbar.iconPromptBox = modelData.icon
                        }
                        onIconStateChanged: {
                            if (modelData.type === LauncherItem.AddIcon) {
                                if (iconState === LauncherItem.Normal) {
                                    item.visible = true
                                    itemName.visible = true
                                    dragMousearea.enabled = true
                                } else {
                                    item.visible = false
                                    itemName.visible = false
                                    dragMousearea.enabled = false
                                }
                            }
                        }
                    }
                    Text {
                        id: itemName
                        anchors.bottom: item.bottom
                        anchors.horizontalCenter: appIcon.horizontalCenter
                        color: "white"
                        text: model.modelData.name
                        elide: Text.ElideRight
                        smooth: true
                        visible: !itemRoot.Drag.active
                    }

                    Component.onCompleted: {
                        if (modelData.type === LauncherItem.Container) {

                        } else if (modelData.type === LauncherItem.Group) {
                            itemName.visible = false
                            var groupcomponent = Qt.createComponent("AppContainer.qml");
                            if (groupcomponent.status === Component.Ready) {
                                var group = groupcomponent.createObject(item, { enable: false, scale: 0.2, transformOrigin: Item.TopLeft,
                                                                            model: modelManager.groupModel(modelData.id) });
                                group.enableItemDrag(true)
                            }
                        } else if (modelData.type === LauncherItem.Widget) {
                            var newObject = Qt.createQmlObject('
                                import QtQuick 2.0
                                import com.kylin.tablet 1.0
                                PluginItem {
                                    id: plugin1
                                    width: 400; height: 648
                                    anchors.centerIn: parent
                                    pluginName: "plugin1"
                                    mapItem: pluginAnchorItem
                                    widgetVisible: !listView.horizontalVelocity
                                    rootWindow: root.rootWindow
                                }',
                                item,
                                "dynamicSnippet1");
                        }
                    }
                }

                Component.onCompleted: {
                    root.z = 1
                    if (viewMoel === null) {
                        console.warn("AppContainer: The model is null, please assign a model!")
                    }
                }
            }
        }
    }
    TaskBar {
        id: taskbar
        parentChange: root
        height: 110 * Style.tabletScaleFactor
        anchors.horizontalCenter: listView.horizontalCenter
        anchors.bottom: listView.bottom
        anchors.bottomMargin: 36 * Style.tabletScaleFactor
        onTaskbarItemEnterDesktop: {
            modelManager.addPlaceholderItem(modelManager.groupPageItemCount(-1, listView.currentIndex), listView.currentIndex)
            taskbarReplayAllowed = true
        }
        onTaskbarItemEnterTaskbar: {
            modelManager.removePlaceholderItem()
            taskbarReplayAllowed = false
        }
    }

    //应用卸载提示框
    InfoBox {
        id: dumpInfoBox; visible: false
        leftBtnText:"取消"; rightBtnText: "确认"
        infoBoxText: taskbar.textPromptBox; infoBoxImageSource: "file://" + taskbar.iconPromptBox
    }
}






