/*
 * Copyright 2022 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

import QtQuick 2.0
import QtQml 2.15
import QtQuick.Layouts 1.15
import QtQuick.Controls 2.5
import QtQml.Models 2.2
import QtQuick.Window 2.3
import com.kylin.tablet 1.0
import MyGeneralSetting 1.0
import "."

Rectangle {
    id: rectRoot
    width: Style.widthScaleFactor * 480;
    height: Style.heightScaleFactor * 200;
    radius: 12;
    color: Qt.rgba(255/255, 255/255, 255/255, 0.75)

    signal logoutEvent

    Image {
        id: loginImage
        anchors.left: parent.left; anchors.leftMargin: Style.widthScaleFactor * 40
        anchors.top: parent.top; anchors.topMargin: Style.heightScaleFactor * 32
        sourceSize.width: Style.widthScaleFactor * 64; sourceSize.height: Style.heightScaleFactor * 64;
        source: "qrc:/ui/images/ukui_portrait_system default.svg"; signal clickEvent(string exec)
        MouseArea{
            anchors.fill: parent
            onClicked: {
                loginImage.clickEvent("ukui-control-center-u")
            }
        }
    }

    Column {
        width: Style.widthScaleFactor * 240;
        anchors.top: parent.top; anchors.topMargin: Style.heightScaleFactor * 34
        anchors.horizontalCenter: parent.horizontalCenter

        Text {
            id: realText
            height: Style.heightScaleFactor * 33; color: "#333333"
            lineHeight: Style.heightScaleFactor * 33; font.pointSize: 18
            font.family: "PingFang SC"; font.styleName: "Regular"
        }
        Text {
            id: userText
            height: Style.heightScaleFactor * 22; color: "#262626"
            lineHeight: Style.heightScaleFactor * 22; font.pointSize: 12
            font.family: "Noto Sans CJK SC Regular"; font.styleName: "Regular"
        }
    }

    Rectangle{
        id: logoutButton
        width: Style.widthScaleFactor * 240; height: Style.heightScaleFactor * 40; radius: 12; border.width: 1
        anchors.top: parent.top; anchors.topMargin: Style.heightScaleFactor * 128
        anchors.horizontalCenter: parent.horizontalCenter
        border.color: "#8C8C8C"; color: "transparent"; opacity: 0.8

        Image {
            id: logoutImage
            sourceSize.width: Style.widthScaleFactor * 24; sourceSize.height: Style.heightScaleFactor * 24
            anchors.left: parent.left; anchors.leftMargin: Style.widthScaleFactor * 68
            anchors.verticalCenter: parent.verticalCenter
            source: "qrc:/ui/images/ukui_depart_default.svg"
        }
        Text {
            id: logoutName
            text: "退出账号"; color: "#333333"; font.pointSize: 14;
            anchors.left: logoutImage.right; anchors.rightMargin: Style.widthScaleFactor * 2
            anchors.verticalCenter: parent.verticalCenter
            font.family: "PingFang SC"; font.styleName: "Regular"
        }
        MouseArea{
            id: mouseArea
            anchors.fill: parent
            onClicked: {
                rectRoot.logoutEvent();
            }
            onPressed:{
                logoutButton.border.color = "#296CD9"
            }
            onReleased: {
                logoutButton.border.color = "#8C8C8C"
            }
        }
    }

    AccountLogout{
        id: accountLogout
    }

    CGeneralSetting {
        id: cGeneralSetting
    }

    function getAccountInfo(realname, username){
        realText.text = realname
        userText.text = username
    }

    Component.onCompleted: {
        //用户注销操作
        rectRoot.logoutEvent.connect(accountLogout.logout)

        //获取用户信息操作
        rectRoot.getAccountInfo(accountLogout.realName,accountLogout.userName)
        accountLogout.userInfoSignal.connect(rectRoot.getAccountInfo)

        //点击头像唤醒用户界面
        loginImage.clickEvent.connect(cGeneralSetting.openGeneralSetting)
    }
}
