/*
 * Copyright 2022 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

import QtQuick 2.15
import QtQuick.Window 2.15
import QtQml.Models 2.15
import QtQml 2.15
import QtGraphicalEffects 1.15
import QtQuick.Controls 2.15
import com.kylin.tablet 1.0
import "."

Rectangle {
    id: taskBar
    property var parentChange
    property int listviewY: 904 * Style.tabletScaleFactor
    property bool replayAllowed
    property bool updateData: false //relese时是否更新数据库

    signal taskbarItemEnterDesktop()
    signal taskbarItemEnterTaskbar()
    property bool taskbarReplayAllowed: false

    property string iconPromptBox
    property string textPromptBox
    property string tarDesktopName

    width: back.width
    height: back.height
    visible: true
    color: "transparent"

    DropArea {
        anchors.fill: parent
        onEntered:  {
            console.log("DropArea onEntered")
            if(listview.count < 6 && drag.source.group !== 1) {
                replayAllowed = true
                modelManager.addPlaceholderItem(listview.count,0,1,1,0)
            }
            if(drag.source.group === 1) {
                taskbarItemEnterTaskbar()
            }
        }
        onExited: {
            console.log("DropArea onExited")
            modelManager.removePlaceholderItem()
            replayAllowed = false
            if(drag.source.group === 1) {
                taskbarItemEnterDesktop()
            }
        }
        Rectangle {
            id: back
            width: listview.width + 186 * Style.tabletScaleFactor//245
            height: 96 * Style.tabletScaleFactor
            radius: 24
            color: "#F0F3F7"
            opacity: 0.85
            anchors.centerIn: parent
        }

        ListView {
            id: listview
            anchors.left: back.left
            anchors.leftMargin: 7.5 * Style.tabletScaleFactor
            anchors.verticalCenter: back.verticalCenter
            width: {
                if(listview.count <= 6) {
                    return 534 * Style.tabletScaleFactor
                }
                if(listview.count > 6 && listview.count <= 15) {
                    return listview.count * 89 * Style.tabletScaleFactor
                }
                if(listview.count > 15) {
                    return (1335 +37) * Style.tabletScaleFactor
                }
            }
            height: back.height * Style.tabletScaleFactor
            clip: true
            orientation: ListView.Horizontal
            addDisplaced: Transition { NumberAnimation { properties: "x,y"; duration: 200 } }
            move: Transition { NumberAnimation { properties: "x,y"; duration: 200 } }
            moveDisplaced: Transition { NumberAnimation { properties: "x,y"; duration: 200 } }
            remove: Transition { NumberAnimation { properties: "x,y"; duration: 200 } }
            removeDisplaced: Transition { NumberAnimation { properties: "x,y"; duration: 200 } }
            displaced: Transition { NumberAnimation { properties: "x,y"; duration: 200 } }
            model: modelManager.taskbarModel
            delegate: listviewDelegate
//            highlightRangeMode: ListView.StrictlyEnforceRange;//切换item的时候currentIndex也跟随着变化
            onMovementStarted: {
                if(listview.count > 15) {
                    leftmask.visible = true
                    rightmask.visible = true
                }
            }
            onMovementEnded: {
                if(listview.count > 15) {
                    leftmask.visible = listview.contentX > 0 ? true : false
                    rightmask.visible = listview.atXEnd ? false : true
                }
            }
            onCountChanged: {
                if(listview.count <= 15) {
                    rightmask.visible = false
                    leftmask.visible = false
                }
                else {
                    leftmask.visible = listview.contentX > 0 ? true : false
                    rightmask.visible = listview.atXEnd ? false : true
                }
            }
        }

        Component {
            id: listviewDelegate

            DropArea {
                id: droparea

                property int group: 1
                property int visualIndex: index
                Binding { target: iconback; property: "visualIndex"; value: visualIndex }

                width: 89 * Style.tabletScaleFactor
                height: back.height * Style.tabletScaleFactor

                onEntered: {
                    updateData = true
                    console.log("taskbar onentered", drag.source.group, visualIndex)
                    if(visualIndex < 0) {
                        return
                    }
                    if (drag.source.group !== group && drag.source.group !== undefined) {
                        modelManager.addPlaceholderItem(visualIndex,0,1,1,0)
                        replayAllowed = true
                    } else {
                        modelManager.moveItem(drag.source.visualIndex, visualIndex, 0, 0)
                        modelManager.removePlaceholderItem();
                    }
                }
                MouseArea {
                    id: dragMousearea
                    anchors.fill: parent
                    onClicked: {
                        modelManager.setItemState(LauncherItem.Normal,0)
                        modelManager.setItemState(LauncherItem.Normal,-1)
                        iconbackMousearea.Drag.active = false
                        iconbackMousearea.drag.target = null
                        dumpInfoBox.visible = false
                    }
                }
                Rectangle {
                    id: iconback
                    property int visualIndex: 0
                    property int group: 1
                    property int pageIndex: 0
                    property int rowSpan: 1
                    property int columnSpan: 1
                    property bool canMove: false

                    width: icon.width * Style.tabletScaleFactor
                    height: icon.height * Style.tabletScaleFactor
                    radius: icon.iconRadius
                    color: "transparent"
                    anchors.horizontalCenter: parent.horizontalCenter
                    anchors.verticalCenter: parent.verticalCenter

                    Drag.active: iconbackMousearea.drag.active
                    Drag.source: iconback
                    Drag.hotSpot.x: iconback.width/2
                    Drag.hotSpot.y: iconback.height/2

                    states: [
                        State {
                            when: iconback.Drag.active
                            ParentChange {
                                target: iconback
                                parent: parentChange
                            }
                            AnchorChanges {
                                target: iconback
                                anchors.horizontalCenter: undefined
                                anchors.verticalCenter: undefined
                            }
                        }
                    ]
                    MouseArea {
                        id: iconbackMousearea
                        anchors.fill: parent
                        hoverEnabled: false
                        pressAndHoldInterval: modelData.state === LauncherItem.Editable ? 100 : undefined
                        onPressAndHold: {
                            iconback.canMove = true
                            if(!modelData.appOpen) {
                                iconbackMousearea.drag.target = iconback
                            }
                            modelManager.setItemState(LauncherItem.Editable, 0)
                            modelManager.setItemState(LauncherItem.Editable, -1)
                        }
                        onClicked: {
                            if(modelData.appOpen && modelData.windowId !== -1) {
                                modelManager.activeWindow(modelData.windowId)
                            }
                            else {
                                if(modelData.state === LauncherItem.Normal){
                                    appManager.runApplication(modelData.desktopFile)
                                }
                            }
                        }
                        onReleased: {
                            icon.iconWidth = 64 * Style.tabletScaleFactor

                            var itemPos = iconback.mapToItem(null,0,0)
                            if(itemPos.y < listviewY && taskbarReplayAllowed) {
                                modelManager.replacePlaceholderItemTaskbar(index,0,0)
                                icon.visible = false
                                taskbarReplayAllowed = false
                            }
                            iconbackMousearea.Drag.active = false
                            iconbackMousearea.drag.target = null
                            iconback.canMove = false
                            if(updateData) {
                                modelManager.updateCachedData(-1)
                                modelManager.updateCachedData(0)
                                updateData = false
                            }
                        }
                        onPositionChanged: {
                            var itemPos = iconback.mapToItem(null,0,0)
                            if(itemPos.y < listviewY) {
                                icon.iconWidth = 96 * Style.tabletScaleFactor
                            }
                            else {
                                icon.iconWidth = 64 * Style.tabletScaleFactor
                            }
                        }
                                                                                                                                                                                                              }
                    AppIcon {
                        id: icon
                        iconWidth: 64 * Style.tabletScaleFactor
                        iconRadius: 15
                        anchors.centerIn: iconback
                        icon: "file://" + modelData.icon
                        visible: modelData.type === LauncherItem.AppIcon
                        iconState: modelData.state
                        opacity: {
                            if (modelData.type === LauncherItem.AppIcon) {
                                if (iconback.canMove) {
                                    return 1
                                }
                                if (iconbackMousearea.pressed) {
                                    return 0.5
                                }
                            }
                            return 1
                        }
                        deleteiconWidth: 28

                        onDeleteIcon: {
                            dumpInfoBox.visible = true
                            dumpInfoBox.desktopFile = modelData.desktopFile
                            dumpInfoBox.visualIndex = index
                            dumpInfoBox.pageIndex = 0
                            dumpInfoBox.groupId = 0

                            taskbar.textPromptBox = model.modelData.name
                            taskbar.iconPromptBox = modelData.icon
//                            visible = false
//                            modelManager.removeItem(index,0,0)
                        }
                    }
                }
                Rectangle {
                    id: openindicator
                    width: 22
                    height: 4 * Style.tabletScaleFactor
                    radius: 2
                    visible: modelData.appOpen
                    anchors.top: iconback.bottom
                    anchors.topMargin: 7 * (1440/1080)//8 * Style.tabletScaleFactor
                    anchors.horizontalCenter: droparea.horizontalCenter
                    antialiasing: true
                    color: "#1890FF"
                    Component.onCompleted: rightmask.visible = listview.count > 15 ? true : false
                }
            }
        }

        Image {
            id: rightmask
            source: "qrc:/ui/images/taskbar-rightmask"
            anchors.verticalCenter: listview.verticalCenter
            anchors.right: listview.right
            visible: false
        }
        Image {
            id: leftmask
            source: "qrc:/ui/images/taskbar-leftmask"
            anchors.left: listview.left
            anchors.verticalCenter: listview.verticalCenter
            visible: false
        }
//        TaskbarMask {
//            width: 10
//            height: 64
//            anchors.verticalCenter: listview.verticalCenter
//            anchors.left: listview.left
//        }
//        TaskbarMask {
//            width: 10
//            height: 64
//            anchors.verticalCenter: listview.verticalCenter
//            anchors.right: listview.right
//            anchors.rightMargin: -1
//            rotation: 180
//        }

//        Image {
//            id: turnpageBtn
//            width: 16 * Style.widthScaleFactor
//            height: 16 * Style.heightScaleFactor
//            anchors.right: rightBack.left
//            anchors.rightMargin: 40 * Style.widthScaleFactor
//            anchors.verticalCenter: rightBack.verticalCenter
//            source: "qrc:/ui/images/rightbtn.png"
//            MouseArea {
//                anchors.fill: parent
//                onClicked: {
//                    listview.highlightMoveDuration = 0
//                    if(listview.currentIndex + 15 <= listview.count) {
//                        listview.currentIndex += 15
//                    }
//                }
//            }
//        }

        Rectangle {
            id: rightBack
            width: 136 * Style.tabletScaleFactor
            height: 64 * Style.tabletScaleFactor
            color: Qt.rgba(0, 0, 0, 0.04)
            radius: 14
            anchors.right: back.right
            anchors.rightMargin: 22.5 * Style.tabletScaleFactor;
            anchors.verticalCenter: back.verticalCenter
            DropArea {
                anchors.fill: parent
                onEntered: {
                    console.log("rightback onentered",modelManager.groupPageItemCount(0,0))
//                    if(modelManager.groupPageItemCount(0,0) === 0) {
//                        replayAllowed = true
//                        modelManager.addPlaceholderItem(0,0,1,1,0)
//                    }
                }
                MouseArea {
                    anchors.fill: parent
                    onPressed: sidebarState.setSidebarShow()
                }
            }
        }

        TimeComponent {
            id: timer
            anchors.verticalCenter: rightBack.verticalCenter
            anchors.left: rightBack.left
            anchors.leftMargin: 14 * Style.tabletScaleFactor
        }
        TrayIcon {
            id: net
            width: 20 * Style.tabletScaleFactor
            height: 20 * Style.tabletScaleFactor
            anchors.top: rightBack.top
            anchors.topMargin: 11 * Style.tabletScaleFactor
            anchors.left: timer.right
            anchors.leftMargin: 16 * Style.tabletScaleFactor
            type: TrayIcon.Wifi
        }
        TrayIcon {
            id: bluetooth
            width: 20 * Style.tabletScaleFactor
            height: 20 * Style.tabletScaleFactor
            anchors.left: net.right
            anchors.leftMargin: 6 * Style.tabletScaleFactor
            anchors.verticalCenter: net.verticalCenter
            type: TrayIcon.Bluetooth
        }

        TrayIcon {
            id: power
            width: 20 * Style.tabletScaleFactor
            height: 20 * Style.tabletScaleFactor
            anchors.bottom: rightBack.bottom
            anchors.bottomMargin: 6 * Style.tabletScaleFactor
            anchors.horizontalCenter: net.horizontalCenter
            type: TrayIcon.Power
        }
        TrayIcon {
            id: information
            width: 20 * Style.tabletScaleFactor
            height: 20 * Style.tabletScaleFactor
            anchors.horizontalCenter: bluetooth.horizontalCenter
            anchors.verticalCenter: power.verticalCenter
            type: TrayIcon.Message
        }
    }

    Component.onCompleted: {
//        net.wifiStateChanged.connect(
//                    function(){
//                        net.update()
//                    }
//                    )
//        sound.soundValueChanged.connect(
//                    function() {
//                        sound.update()
//                    }
//                    )
//        power.powerValueChanged.connect(
//                    function() {
//                        power.update()
//                    }
//                    )
//        information.messageStateChanged.connect(
//                    function() {
//                        information.update()
//                    }
//                    )
    }
}
