/*
 * Copyright 2022 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

import QtQuick 2.15
import QtQuick.Layouts 1.15
import "."

Rectangle {
    id: root
    property alias title: titleInput.text
    property bool readOnly: true
    property bool isEditing: false
    property var txtwidth: root.width
    property var titleTxt: titleInput.text
    property var lastWidth

    signal editVisible()
    signal editInVisible()

    width: titleInput.contentWidth < Style.widthScaleFactor * 1 ?
                                     Style.widthScaleFactor * 70 : titleInput.contentWidth + image.width + Style.widthScaleFactor * 25
    height: Style.heightScaleFactor * 40
    radius: 10
    color: "white"

    TextInput {
        id: titleInput
        anchors.fill: parent
        padding: Style.widthScaleFactor * 7
        horizontalAlignment: TextInput.AlignHCenter
        verticalAlignment: TextInput.AlignVCenter
        maximumLength: 8
        font.pointSize: 14
        clip: true
        readOnly: true
        focus: false
        text: title
        rightPadding: Style.widthScaleFactor * 16

        MouseArea{
            anchors.fill: parent
            onClicked: {
                if (titleEditable === false){
                    container.focusChange()
                    container.clicked()
                }
            }
            onPressAndHold: {
                //不同Container是否可编辑状态的判断
                //titleEditable为true是为可以编辑的状态
                if (titleEditable === true && image.state === "Normal"){
                    if (image.state === "Normal") {
                        titleInput.readOnly = false
                        titleInput.focus = true
                        titleInput.horizontalAlignment = TextInput.AlignLeft
                        titleInput.rightPadding = image.width + Style.heightScaleFactor * 20
                        image.state = "Editable"
                        if(image.state === "Editable"){
                            root.width = Style.widthScaleFactor * 400
                            isEditing = true
                            root.editVisible()
                            title = titleInput.text
                            if(title == ""){
                                title = root.titleTxt
                            }
                            else{
                                 root.titleTxt = title
                            }
                        }
                    }
                    else {
                        titleInput.clear()
                        titleText.Layout.preferredWidth = Style.widthScaleFactor * 400
                    }
                }
            }
        }

        Image {
            id: image
            visible: !readOnly
            anchors { right: titleInput.right; rightMargin: Style.widthScaleFactor * 5 }

            y: (root.height - image.sourceSize.height) / 2
            sourceSize { width: Style.widthScaleFactor * 16; height: Style.heightScaleFactor * 16; }
            state: "Normal"
            states: [
                State {
                    name: "Normal"
                    PropertyChanges {
                        target: image
                        source: "qrc:/ui/images/ukui_teaching_default.svg"
                    }
                },
                State {
                    name: "Editable"
                    PropertyChanges {
                        target: image
                        source: "qrc:/ui/images/ukui_exit_symbolic.svg"
                    }
                }
            ]
            MouseArea {
                anchors.fill: parent
                onClicked: {
                    if (image.state === "Normal") {
                        titleInput.readOnly = false
                        titleInput.focus = true
                        titleInput.horizontalAlignment = TextInput.AlignLeft
                        titleInput.rightPadding = image.width + Style.widthScaleFactor * 20
                        image.state = "Editable"

                        if(image.state === "Editable"){
                            root.width = Style.widthScaleFactor * 400
                            isEditing = true
                            root.editVisible()
                            title = titleInput.text
                            if(title == ""){
                                title = root.titleTxt
                            }
                            else{
                                 root.titleTxt = title
                            }
                        }
                    }
                    else {
                        titleInput.clear()
                        root.width = Style.widthScaleFactor * 400
                    }
                }
            }
        }

        onEditingFinished: {
            readOnly = true
            focus = false
            titleInput.horizontalAlignment = TextInput.AlignHCenter
            titleInput.rightPadding = Style.widthScaleFactor * 20
            title = titleInput.text
            image.state = "Normal"
            isEditing = false
            root.width = titleInput.contentWidth < Style.widthScaleFactor * 1 ?
                         Style.widthScaleFactor * 70 : titleInput.contentWidth + image.width + Style.widthScaleFactor * 25
            root.editInVisible()
            root.lastWidth = root.width
            if(text == ""){
                text = root.titleTxt
                root.width = titleInput.contentWidth < Style.widthScaleFactor * 1 ?
                             Style.widthScaleFactor * 70 : titleInput.contentWidth + image.width + Style.widthScaleFactor * 25
                root.editInVisible()
                root.lastWidth = root.width
            }
            else{
                 root.titleTxt = text
            }
        }
    }

    function focusHandle(){
        container.focus = true
    }
}
