/*
 * Copyright 2022 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

import QtQuick 2.15
import QtQuick.Window 2.15
import QtQuick.Controls 2.15
import QtQml 2.15
import com.kylin.tablet 1.0
import "."

TabletDesktopWindow {
    id: root
    visibility: Window.FullScreen
    visible: true

    MouseArea{
        anchors.fill: parent
        onClicked: {
            itemView.titleFocusChanged()
        }
    }

    TimeComponent {
        id: dateTime
        x: itemView.x + Style.widthScaleFactor * 20
        y: Style.heightScaleFactor * 68
    }

    LogoComponent {
        x: Style.widthScaleFactor * 1550; y: Style.heightScaleFactor * 84
    }

    NetworkState{
        id: networkState
        anchors.top: parent.top
        anchors.topMargin: Style.widthScaleFactor * 8
        anchors.right: parent.right
        anchors.rightMargin: Style.widthScaleFactor * 10
    }

    LogoTopComponent {
        id: logoTop
        anchors.bottom: itemView.top
        anchors.bottomMargin: Style.widthScaleFactor * 20
        anchors.right: itemView.right
        anchors.rightMargin: Style.widthScaleFactor * 20
    }

    LauncherView {
        id: itemView
        property real margin: Style.widthScaleFactor * (200-20)
        property real bottomMargin: Style.heightScaleFactor * (160-20)
        anchors {
            centerIn: parent
            topMargin: margin; leftMargin: margin;
            rightMargin: margin; bottomMargin: bottomMargin
        }
        width: Style.widthScaleFactor * (1520 + 40);
        height: Style.heightScaleFactor * (720 + 40)
        viewMoel: modelManager.launcherModel
        onShowAddAppWindow: {
            if (groupId === 4) {
                modelManager.setAddAppCategory(CategoryItem.All, groupId)
                addAppWindow.setClassifyId = CategoryItem.All
            } else if (groupId === 2) {
                modelManager.setAddAppCategory(CategoryItem.Education, groupId)
                addAppWindow.setClassifyId = CategoryItem.Education
            }
            addAppWindow.groupId = groupId
            addAppWindow.groupPageIndex = pageIndex
            addAppWindow.showFullScreen()
        }
    }

    LogoBottomComponent{
        anchors.bottom: parent.bottom
        anchors.bottomMargin: Style.widthScaleFactor * 58
        anchors.right: parent.right
        anchors.rightMargin: Style.widthScaleFactor * 80
    }

    GeneralSettingButton {
        dialogWidth: Style.widthScaleFactor * 64; dialogHeight: Style.widthScaleFactor * 64
        imageWidth: Style.widthScaleFactor * 40; imageHeight: Style.widthScaleFactor * 40
        x: Style.widthScaleFactor * 80
        y: Style.heightScaleFactor * 976
        imagePath: "qrc:/ui/images/document-properties-symbolic.svg"
        execName: "ukui-control-center.desktop"
        dialogOpacity: 0.4
    }

    GeneralSettingButton {
        dialogWidth:  Style.widthScaleFactor * 96; dialogHeight: Style.widthScaleFactor * 96
        imageWidth:  Style.widthScaleFactor * 64; imageHeight: Style.widthScaleFactor * 40
        x: Style.widthScaleFactor * 49
        y: Style.heightScaleFactor * 480
        imagePath: "qrc:/ui/images/ukui_keyboard-symbolic.svg"
        execName: "onboard"
        dragSwitch: true
    }

    AddApp {
        id: addAppWindow
        visible: false
    }
}
