/*
 * Copyright 2022 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

import QtQuick 2.15
import QtQuick.Window 2.15
import QtQuick.Controls 2.15
import QtQml 2.15
import com.kylin.tablet 1.0
import QtQuick.Layouts 1.15
import "."

TabletDesktopWindow {
    id: rooWindow
    objectName: "rootRectangle"
    // anchors.fill: parent
    visible: true
    state: "Hide"

    Rectangle {
        id: rootContent
        objectName: "contentRectangle"
        anchors.fill: parent
        color: "transparent"

        Item {
            id: pluginAnchorsItem
            anchors.fill: parent
        }

        LauncherView {
            id: itemView
            pluginAnchorItem: pluginAnchorsItem
            rootWindow: rootContent
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.fill: parent
            viewMoel: modelManager.launcherModel
        }

        PageSwitchIndicator {
            id: pageIndicator
            visible: itemView.listViewItemCount() > 1
            anchors.bottom: parent.bottom
            y: 0
            anchors.bottomMargin: 176 * Style.tabletScaleFactor//parent.height / (Screen.width > Screen.height ? 4 : 5)
            anchors.horizontalCenter: parent.horizontalCenter
            pageContainer: itemView.pageContainer
        }
    }

    states: [
        State {
            name: "Show"
            PropertyChanges {
                target: rootContent
                opacity: 1
                scale: 1
            }
        },
        State {
            name: "Hide"
            PropertyChanges {
                target: rootContent
                opacity: 0
                scale: 0.9
            }
        }
    ]
    transitions: Transition {
        ParallelAnimation {
            NumberAnimation { target: rootContent; properties: "width,height,opacity,scale"; duration: 200 }
        }
    }
    Component.onCompleted: {
    }

}


