/*
 * Copyright 2022 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

import QtQuick 2.15
import QtQuick.Window 2.15
import QtQml.Models 2.1

Window {
    id: root
    width: 640
    height: 480
    visible: true
    title: qsTr("Hello World")

    Timer {
        id: timer
        repeat: false
        interval: 2000
    }

    Item {
        id: rootItem
        z: 100
        anchors.fill: parent
    }

    Rectangle {
        id: fullScreenGroup
        visible: false
        anchors.fill: parent
        z: 99
        color: "white"
        //弹出的绿色矩形框
        Rectangle {
            anchors.centerIn: parent
            width: 200; height: 200
            color: "green"
            radius: 5
            DropArea {
                property bool hasEntered: false
                anchors.fill: parent
                onEntered: {
                    hasEntered = true
                }
                onExited: {
                    if (hasEntered) {
                        fullScreenGroup.visible = false
                    }
                }
            }
        }
    }

    GridView {
        id: gridView
        anchors.fill: parent
        displaced: Transition {
            NumberAnimation { properties: "x,y"; easing.type: Easing.OutQuad }
        }

        cellWidth: 80; cellHeight: 80

        model: DelegateModel {
            id: visualModel
            model: ListModel {
                id: colorModel
                ListElement { color: "blue" }
                ListElement { color: "green" }
                ListElement { color: "red" }
                ListElement { color: "yellow" }
                ListElement { color: "orange" }
                ListElement { color: "purple" }
                ListElement { color: "cyan" }
                ListElement { color: "magenta" }
                ListElement { color: "chartreuse" }
                ListElement { color: "aquamarine" }
                ListElement { color: "indigo" }
                ListElement { color: "black" }
                ListElement { color: "lightsteelblue" }
                ListElement { color: "violet" }
                ListElement { color: "grey" }
                ListElement { color: "springgreen" }
                ListElement { color: "salmon" }
                ListElement { color: "blanchedalmond" }
                ListElement { color: "forestgreen" }
                ListElement { color: "pink" }
                ListElement { color: "navy" }
                ListElement { color: "goldenrod" }
                ListElement { color: "crimson" }
                ListElement { color: "teal" }
            }
            delegate: DropArea {
                id: dropArea
                property int oriIndex: -1
                function onCoverTimeout() {
                    if (dropArea.drag.source) {
                        fullScreenGroup.visible = true
                    }
                }
                width: 70; height: 70
                onEntered: {
                    console.log("onEntered")

                    if (index !== oriIndex)
                    {
                        backgroundItem.color = "lightgray"
                    }
                    timer.start()
                    dropArea.oriIndex = index;
                }
                onExited: {
                    console.log("onExited")
                    backgroundItem.color = "transparent"
                    timer.stop()
                }



                Rectangle {
                    id: backgroundItem
                    anchors.fill: parent
                    color: "transparent"

                    Rectangle {
                        id: visibleItem
                        width: 50; height: 50
                        anchors {
                            horizontalCenter: parent.horizontalCenter;
                            verticalCenter: parent.verticalCenter
                        }

                        color: model.color
                        radius: 3

                        DragHandler {
                            id: dragHandler
                        }
                        Drag.active: dragHandler.active
                        Drag.source: visibleItem
                        Drag.hotSpot.x: 35
                        Drag.hotSpot.y: 35

                        states: [
                            State {
                                when: visibleItem.Drag.active
                                ParentChange {
                                    target: visibleItem
                                    parent: rootItem
                                }

                                PropertyChanges {
                                    target: visibleItem
                                    z: 101
                                }
                                PropertyChanges {
                                    target: dropArea
                                    oriIndex: index
                                }

                                AnchorChanges {
                                    target: visibleItem
                                    anchors.horizontalCenter: undefined
                                    anchors.verticalCenter: undefined
                                }
                            }
                        ]
                    }
                }
                Component.onCompleted: {
                    timer.triggered.connect(dropArea.onCoverTimeout)
                }
            }
        }
    }
}
